@@ -5,11 +5,11 @@ _LIBTOOL_ARCHIVE="libtool-2.4.6.tar.xz"
 
 _LIBICONV_ARCHIVE="libiconv-1.14.tar.gz"
 _LIBXML2_ARCHIVE="libxml2-2.9.3.tar.gz"
-_GETTEXT_ARCHIVE="gettext-0.19.6.tar.xz"
+_GETTEXT_ARCHIVE="gettext-0.19.7.tar.xz"
 
-_LIBCURL_ARCHIVE="curl-7.46.0.tar.gz"
-_OPENSSL_ARCHIVE="openssl-1.0.2e.tar.gz"
+_LIBCURL_ARCHIVE="curl-7.47.1.tar.gz"
+_OPENSSL_ARCHIVE="openssl-1.0.2f.tar.gz"
 
-_LIBISDS_ARCHIVE="libisds-0.10.1.tar.xz"
+_LIBISDS_ARCHIVE="libisds-0.10.1.tar.xz" # 10.2 is available but we have not tested it
 _LIBISDS_GIT="https://gitlab.labs.nic.cz/kslany/libisds.git"
 #_LIBISDS_BRANCH="feature-openssl" # Use master.
