@@ -27,15 +27,20 @@
 #include "src/gui/datovka.h"
 #include "src/io/account_db.h"
 #include "src/io/dbs.h"
+#include "src/io/filesystem.h"
 #include "src/io/isds_sessions.h"
 #include "src/log/log.h"
-#include "src/thread/worker.h"
+#include "src/worker/pool.h"
+#include "src/worker/task_download_message.h"
+#include "src/worker/task_download_message_list.h"
+#include "src/worker/task_search_owner.h"
 
 // Known attributes definition
 const QStringList connectAttrs = QStringList()
     << "username" << "password" << "certificate" << "otpcode";
 const QStringList getMsgListAttrs = QStringList()
-    << "dmType" << "dmStatusFilter" << "dmLimit" << "dmFromTime" << "dmToTime";
+    << "dmType" << "dmStatusFilter" << "dmLimit" << "dmFromTime" << "dmToTime"
+    << "complete";
 const QStringList sendMsgAttrs = QStringList()
     << "dbIDRecipient" << "dmAnnotation" << "dmToHands"
     << "dmRecipientRefNumber" << "dmSenderRefNumber" << "dmRecipientIdent"
@@ -53,6 +58,7 @@ const QStringList findDataboxAttrs = QStringList() << "dbType" << "dbID"
 
 
 /* ========================================================================= */
+static
 void printDataToStdOut(const QStringList &data)
 /* ========================================================================= */
 {
@@ -62,11 +68,24 @@ void printDataToStdOut(const QStringList &data)
 		if (i == (data.count() - 1)) {
 			cout << data.at(i) << endl << endl;
 		} else {
-			cout << data.at(i) + " ";
+			cout << data.at(i) << " ";
 		}
 	}
 }
 
+static
+void printDataToStdOut(const QList<qint64> &data)
+{
+	QTextStream cout(stdout);
+
+	for (int i = 0; i < data.count(); ++i) {
+		if (i == (data.count() - 1)) {
+			cout << data.at(i) << endl << endl;
+		} else {
+			cout << data.at(i) << " ";
+		}
+	}
+}
 
 /* ========================================================================= */
 void printErrToStdErr(const cli_error err, const QString errmsg)
@@ -149,12 +168,9 @@ cli_error getMsgList(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 	    << username;
 
 	/* messages counters */
-	int st, sn, rt, rn;
-	st = sn = rt = rn = 0;
-	qdatovka_error ret;
-	QStringList newMsgIdList;
+	QList<qint64> newMsgIdList;
+	bool complete = false;
 	unsigned long dmLimit = 0;
-	unsigned long *dmLimitPtr = NULL;
 	uint dmStatusFilter = MESSAGESTATE_ANY;
 	bool ok;
 
@@ -181,6 +197,17 @@ cli_error getMsgList(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 		}
 	}
 
+	if (map.contains("complete")) {
+		QString compValue = map.value("complete").toString();
+		if (!(compValue == "no") && !(compValue == "yes")) {
+			errmsg = "complete attribute has wrong value "
+			    "(no,yes is required)";
+			qDebug() << createErrorMsg(errmsg);
+			return CLI_ATR_VAL_ERR;
+		}
+		complete = (compValue == "yes") ? true : false;
+	}
+
 	if (map.contains("dmLimit")) {
 		dmLimit = map["dmLimit"].toString().toULong(&ok);
 		if (!ok) {
@@ -189,88 +216,96 @@ cli_error getMsgList(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 			qDebug() << CLI_PREFIX << errmsg;
 			return CLI_ATR_VAL_ERR;
 		}
-		dmLimitPtr = &dmLimit;
 	}
 
-	if (NULL == dmLimitPtr) {
+	if (0 == dmLimit) {
+		/* Increase limit. */
 		dmLimit = MESSAGE_LIST_LIMIT;
-		dmLimitPtr = &dmLimit;
 	}
 
-	QString err;
-	if (map["dmType"].toString() == MT_RECEIVED) {
-
-		ret = Worker::downloadMessageList(username, MSG_RECEIVED,
-		    *msgDbSet, err, NULL, 0, 0, rt, rn, newMsgIdList,
-		    dmLimitPtr, dmStatusFilter);
-
-		if (Q_SUCCESS == ret) {
-			qDebug() << CLI_PREFIX << "Received message list "
-			    "has been downloaded";
-			printDataToStdOut(newMsgIdList);
-			return CLI_SUCCESS;
-		} else {
-			errmsg = "Error while downloading received message "
-			    "list";
-			qDebug() << CLI_PREFIX << errmsg << "Error code:"
-			    << ret << err;
-			return CLI_ERROR;
-		}
-
-	} else if (map["dmType"].toString() == MT_SENT) {
-
-		ret = Worker::downloadMessageList(username, MSG_SENT,
-		    *msgDbSet, err, NULL, 0, 0, st, sn, newMsgIdList,
-		    dmLimitPtr, dmStatusFilter);
-
-		if (Q_SUCCESS == ret) {
-			qDebug() << CLI_PREFIX << "Sent message list has been "
-			    "downloaded";
-			printDataToStdOut(newMsgIdList);
-			return CLI_SUCCESS;
-		} else {
-			errmsg = "Error while downloading sent message list";
-			qDebug() << CLI_PREFIX << errmsg << "Error code:"
-			    << ret << err;
-			return CLI_ERROR;
-		}
-
-	} else if (map["dmType"].toString() == MT_SENT_RECEIVED) {
-
-		cli_error lret = CLI_SUCCESS;
-		ret = Worker::downloadMessageList(username, MSG_RECEIVED,
-		    *msgDbSet, err, NULL, 0, 0, rt, rn, newMsgIdList,
-		    dmLimitPtr, dmStatusFilter);
-		if (Q_SUCCESS == ret) {
-			qDebug() << CLI_PREFIX << "Received message list has "
-			    "been downloaded";
-		}  else {
-			errmsg = "Error while downloading received message "
-			    "list";
-			qDebug() << CLI_PREFIX << errmsg << "Error code:" <<
-			    ret << err;
-			lret = CLI_ERROR;
-		}
-		ret = Worker::downloadMessageList(username, MSG_SENT,
-		    *msgDbSet, errmsg, NULL, 0, 0, st, sn, newMsgIdList,
-		    dmLimitPtr, dmStatusFilter);
-		if (Q_SUCCESS == ret) {
-			qDebug() << CLI_PREFIX << "Sent message list has been "
-			    "downloaded";
-		}  else {
-			errmsg = "Error while downloading sent message list";
-			qDebug() << CLI_PREFIX << errmsg << "Error code:"
-			    << ret << err;
-			lret = CLI_ERROR;
-		}
-		printDataToStdOut(newMsgIdList);
-		return lret;
-
-	} else {
+	if ((map["dmType"].toString() != MT_RECEIVED) &&
+	    (map["dmType"].toString() != MT_SENT) &&
+	    (map["dmType"].toString() != MT_SENT_RECEIVED)) {
 		errmsg = "Wrong dmType value: " + map["dmType"].toString();
 		qDebug() << CLI_PREFIX << errmsg;
 		return CLI_ATR_VAL_ERR;
 	}
+
+	QString err, longErr;
+	if ((map["dmType"].toString() == MT_RECEIVED) ||
+	    (map["dmType"].toString() == MT_SENT_RECEIVED)) {
+		TaskDownloadMessageList *task;
+
+		task = new (std::nothrow) TaskDownloadMessageList(
+		    username, msgDbSet, MSG_RECEIVED, complete, dmLimit,
+		    dmStatusFilter);
+		task->setAutoDelete(false);
+		globWorkPool.runSingle(task);
+
+		bool success =
+		   TaskDownloadMessageList::DL_SUCCESS == task->m_result;
+
+		if (success) {
+			newMsgIdList += task->m_newMsgIdList;
+			qDebug() << CLI_PREFIX <<
+			    "Received message list has been downloaded";
+		} else {
+			errmsg =
+			    "Error while downloading received message list";
+			qDebug() << CLI_PREFIX << errmsg << "Error code:" <<
+			    task->m_result << task->m_isdsError <<
+			    task->m_isdsLongError;
+		}
+
+		delete task;
+
+		if (!success) {
+			/* Stop pending jobs. */
+			globWorkPool.stop();
+			globWorkPool.clear();
+			return CLI_ERROR;
+		}
+	}
+
+	if ((map["dmType"].toString() == MT_SENT) ||
+	    (map["dmType"].toString() == MT_SENT_RECEIVED)) {
+		TaskDownloadMessageList *task;
+
+		task = new (std::nothrow) TaskDownloadMessageList(
+		    username, msgDbSet, MSG_SENT, complete, dmLimit,
+		    dmStatusFilter);
+		task->setAutoDelete(false);
+		globWorkPool.runSingle(task);
+
+		bool success =
+		   TaskDownloadMessageList::DL_SUCCESS == task->m_result;
+
+		if (success) {
+			newMsgIdList += task->m_newMsgIdList;
+			qDebug() << CLI_PREFIX <<
+			    "Sent message list has been downloaded";
+		} else {
+			errmsg = "Error while downloading sent message list";
+			qDebug() << CLI_PREFIX << errmsg << "Error code:" <<
+			    task->m_result << task->m_isdsError <<
+			    task->m_isdsLongError;
+		}
+
+		delete task;
+
+		if (!success) {
+			/* Stop pending jobs. */
+			globWorkPool.stop();
+			globWorkPool.clear();
+			return CLI_ERROR;
+		}
+	}
+
+	/* Wait for possible pending jobs. */
+	globWorkPool.wait();
+
+	printDataToStdOut(newMsgIdList);
+	return CLI_SUCCESS;
 }
 
 
@@ -282,8 +317,8 @@ cli_error getMsg(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 	qDebug() << CLI_PREFIX << "Downloading of message" <<
 	    map["dmID"].toString();
 
-	QString err;
-	qdatovka_error ret;
+	QString err, longErr;
+	TaskDownloadMessage::Result ret;
 	const QString username = map["username"].toString();
 
 	MessageDb::MsgId msgId =
@@ -305,11 +340,11 @@ cli_error getMsg(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 	if (needsISDS) {
 		if (map["dmType"].toString() == MT_RECEIVED) {
 
-			ret = Worker::downloadMessage(username,
+			ret = TaskDownloadMessage::downloadMessage(username,
 			    msgId, true, MSG_RECEIVED,
-			    *msgDbSet, err, NULL, 0, 0);
+			    *msgDbSet, err, longErr, NULL);
 
-			if (Q_SUCCESS == ret) {
+			if (TaskDownloadMessage::DM_SUCCESS == ret) {
 				qDebug() << CLI_PREFIX << "Received message" <<
 				    map["dmID"].toString() << "has been "
 				    "downloaded";
@@ -323,11 +358,11 @@ cli_error getMsg(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 
 		} else if (map["dmType"].toString() == MT_SENT) {
 
-			ret = Worker::downloadMessage(username,
+			ret = TaskDownloadMessage::downloadMessage(username,
 			    msgId, true, MSG_SENT,
-			    *msgDbSet, err, NULL, 0, 0);
+			    *msgDbSet, err, longErr, NULL);
 
-			if (Q_SUCCESS == ret) {
+			if (TaskDownloadMessage::DM_SUCCESS == ret) {
 				qDebug() << CLI_PREFIX << "Sent message" <<
 				    map["dmID"].toString() << "has been "
 				    "downloaded";
@@ -335,7 +370,7 @@ cli_error getMsg(const QMap<QString,QVariant> &map, MessageDbSet *msgDbSet,
 				errmsg = "Error while downloading sent "
 				    "message";
 				qDebug() << CLI_PREFIX << errmsg <<
-				    "Error code:" << ret << err;
+				    "Error code:" << ret << err << longErr;
 				return CLI_ERROR;
 			}
 		} else {
@@ -464,10 +499,14 @@ cli_error getDeliveryInfo(const QMap<QString,QVariant> &map,
 	}
 
 	if (needsISDS) {
-		if (Worker::getDeliveryInfo(username,
-		    map["dmID"].toLongLong(), true, *msgDbSet)) {
-			qDebug() << CLI_PREFIX << "Delivery info of message" <<
-			    map["dmID"].toString() << "has been downloaded.";
+		QString isdsError, isdsLongError;
+		if (TaskDownloadMessage::DM_SUCCESS ==
+		    TaskDownloadMessage::downloadDeliveryInfo(username,
+		        map["dmID"].toLongLong(), true, *msgDbSet, isdsError,
+		        isdsLongError)) {
+			qDebug() << CLI_PREFIX << "Delivery info of message"
+			    << map["dmID"].toString() << "has been downloaded."
+			    << isdsError << isdsLongError;
 		} else {
 			errmsg = "Error while downloading delivery info";
 			qDebug() << CLI_PREFIX << errmsg;
@@ -640,7 +679,7 @@ cli_error findDatabox(const QMap <QString, QVariant> &map, QString &errmsg)
 		return CLI_ERROR;
 	}
 
-	isds_error status = isdsSearch(&boxes, username, ownerInfo);
+	int status = TaskSearchOwner::isdsSearch(username, ownerInfo, &boxes);
 	isds_DbOwnerInfo_free(&ownerInfo);
 
 	session = isdsSessions.session(username);
