@@ -310,7 +310,7 @@ int crypto_init(void)
 	const struct pem_str *pem_desc;
 	int loaded_at_once;
 #define MAX_PATH_LEN 256
-	size_t crt_dir_path_len, file_name_len;
+	size_t file_name_len;
 	char file_path[MAX_PATH_LEN];
 
 	debug_func_call();
@@ -335,21 +335,22 @@ int crypto_init(void)
 	/* Load from files. */
 	pem_file = pem_files;
 	assert(NULL != pem_file);
+	/* The prefix doesn't change. */
+	const char *crt_dir = CERT_PATH_PREFIX CERT_DIR;
+	const size_t crt_dir_path_len = strlen(crt_dir);
+	if (crt_dir_path_len >= MAX_PATH_LEN) {
+		log_error("File path buffer is too short for '%s'.\n",
+		    crt_dir);
+		goto skip_files;
+	}
+	memcpy(file_path, crt_dir, crt_dir_path_len);
+	file_path[crt_dir_path_len] = '\0';
 	while (NULL != *pem_file) {
-		/* Construct full file name. */
-		crt_dir_path_len = strlen(CERT_PATH_PREFIX CERT_DIR);
-		if (crt_dir_path_len >= MAX_PATH_LEN) {
-			log_error("File path buffer is to short for '%s%s'.\n",
-			    CERT_PATH_PREFIX, CERT_DIR);
-			continue;
-		}
-		memcpy(file_path, CERT_PATH_PREFIX CERT_DIR, crt_dir_path_len);
-		file_path[crt_dir_path_len] = '\0';
+		/* Append file name to construct full file name. */
 		file_name_len = strlen(*pem_file);
 		if ((crt_dir_path_len + file_name_len) >= MAX_PATH_LEN) {
-			log_error(
-			    "File path buffer is to short for '%s%s%s'.\n",
-			    CERT_PATH_PREFIX, CERT_DIR, *pem_file);
+			log_error("File path buffer is too short for '%s%s'.\n",
+			    crt_dir, *pem_file);
 			continue;
 		}
 		memcpy(file_path + crt_dir_path_len, *pem_file, file_name_len);
@@ -364,6 +365,7 @@ int crypto_init(void)
 		}
 		++pem_file;
 	}
+skip_files:
 
 	/* Load from built-in certificates. */
 	pem_desc = pem_strs;
@@ -512,7 +514,7 @@ int raw_msg_verify_signature(const void *der, size_t der_size, int verify_cert,
 	cms = cms_load_der(der, der_size);
 	if (NULL == cms) {
 		log_error("%s\n", "Could not load CMS.");
-		goto fail;
+		return -1;
 	}
 
 	ret = cms_verify_signature(cms,
@@ -529,12 +531,6 @@ int raw_msg_verify_signature(const void *der, size_t der_size, int verify_cert,
 
 	return ret;
 
-fail:
-	if (NULL != cms) {
-		CMS_ContentInfo_free(cms);
-	}
-	return -1;
-
 #ifdef PRINT_HANDLED_CERT
 #  undef PRINT_HANDLED_CERT
 #endif /* PRINT_HANDLED_CERT */
@@ -1480,9 +1476,11 @@ int x509_store_add_cert_file(X509_STORE *store, const char *fname,
 		*num_loaded = stored_pems;
 	}
 
+	/* stored_pems cannot be 0 here
 	if (0 == stored_pems) {
 		goto fail;
 	}
+	*/
 
 	return 0;
 
@@ -1490,9 +1488,11 @@ fail:
 	if (NULL != bio) {
 		BIO_free(bio);
 	}
+	/* x509 cannot be NULL here
 	if (NULL != x509) {
 		X509_free(x509);
 	}
+	*/
 	return -1;
 
 #ifdef PRINT_HANDLED_CERT
