@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 CZ.NIC
+ * Copyright (C) 2014-2016 CZ.NIC
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,9 +29,9 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QMainWindow>
+#include <QSet>
 #include <QStandardItemModel>
 #include <QProgressBar>
-#include <QThread>
 #include <QTimer>
 #include <QPushButton>
 #include <QNetworkReply>
@@ -44,7 +44,9 @@
 #include "src/gui/dlg_timestamp_expir.h"
 #include "src/models/accounts_model.h"
 #include "src/models/sort_filter_proxy_model.h"
-#include "thread/worker.h"
+#include "src/settings/preferences.h"
+#include "src/single/single_instance.h"
+#include "src/worker/task.h" /* TODO -- remove this header file. */
 
 
 namespace Ui {
@@ -59,30 +61,6 @@ public:
 	explicit MainWindow(QWidget *parent = 0);
 	~MainWindow(void);
 
-	void setDefaultProgressStatus(void);
-
-	QProgressBar *m_statusProgressBar; /*!< Progress-bar object. */
-	QLabel *statusOnlineLabel;
-	QLabel *statusDbMode;
-	QStatusBar *statusBar;
-
-	/* tmp account info struct for ZFO import */
-	class AccountDataStruct {
-	public:
-		QString databoxID;
-		QString accountName;
-		QString username;
-		MessageDbSet *messageDbSet;
-		QModelIndex acntIndex;
-	};
-
-	enum isdsResult {
-		MSG_IS_IN_ISDS,
-		MSG_IS_NOT_IN_ISDS,
-		MSG_ISDS_ERROR,
-		MSG_FILE_ERROR
-	};
-
 	/*!
 	 * @brief Create configuration file if not present.
 	 */
@@ -146,10 +124,35 @@ protected:
 private slots:
 
 	/*!
-	 * @brief Clear info status bar if download of complete message fails.
+	 * @brief Processes messages from single instance emitter.
 	 */
-	void clearInfoInStatusBarAndShowDialog(qint64 msgId,
-	    const QString &errMsg);
+	void processSingleInstanceMessages(const QString &message);
+
+	/*!
+	 * @brief Workers finished.
+	 */
+	void workersFinished(void);
+
+	/*!
+	 * @brief Performs action depending on message download outcome.
+	 */
+	void collectDownloadMessageStatus(const QString &usrName, qint64 msgId,
+	    const QDateTime &deliveryTime, int result, const QString &errDesc);
+
+	/*!
+	 * @brief Performs action depending on message list download outcome.
+	 */
+	void collectDownloadMessageListStatus(const QString &usrName,
+	    int direction, int result, const QString &errDesc,
+	    bool add, int rt, int rn, int st, int sn);
+
+	/*!
+	 * @brief Performs action depending on message send outcome.
+	 */
+	void collectSendMessageStatus(const QString &userName,
+	    const QString &transactId, int result, const QString &resultDesc,
+	    const QString &dbIDRecipient, const QString &recipientName,
+	    bool isPDZ, qint64 dmId);
 
 	/*!
 	 * @brief Version response slot.
@@ -224,10 +227,10 @@ private slots:
 	    qint64 msgId, const QString &deliveryYear, int msgType);
 
 	/*!
-	 * @brief Redraws widgets according to selected attachment item.
+	 * @brief Redraws widgets according to attachment item selection.
 	 */
-	void attachmentItemCurrentChanged(const QModelIndex &current,
-	    const QModelIndex &previous = QModelIndex());
+	void attachmentItemsSelectionChanged(const QItemSelection &selected,
+	    const QItemSelection &deselected = QItemSelection());
 
 	/*!
 	 * @brief Generates menu to selected message item.
@@ -241,9 +244,9 @@ private slots:
 	void attachmentItemDoubleClicked(const QModelIndex &index);
 
 	/*!
-	 * @brief Saves selected attachment to file.
+	 * @brief Saves selected attachments to file.
 	 */
-	void saveSelectedAttachmentToFile(void);
+	void saveSelectedAttachmentsToFile(void);
 
 	/*!
 	 * @brief Save all attachments to dir.
@@ -385,16 +388,6 @@ private slots:
 	 */
 	void downloadSelectedMessageAttachments(void);
 
-	/*!
-	 * @brief Process pending worker jobs.
-	 */
-	void processPendingWorkerJobs(void);
-
-	/*!
-	 * @brief End current worker job.
-	 */
-	void endCurrentWorkerJob(void);
-
 	/*!
 	 * @brief Create and send a new message form selected account.
 	 */
@@ -481,33 +474,40 @@ private slots:
 	void showSignatureDetails(void);
 
 	/*!
-	 * @brief Export message into as ZFO file dialog.
+	 * @brief Export selected message into ZFO files.
 	 */
-	void exportSelectedMessageAsZFO(const QString &attachPath = QString(),
-	    QString userName = QString(), qint64 dmID = -1,
-	    const QDateTime &delivTime = QDateTime());
+	void exportSelectedMessagesAsZFO(void);
 
 	/*!
-	 * @brief Export delivery information as ZFO file dialog.
+	 * @brief Export selected delivery information as ZFO files.
 	 */
-	void exportDeliveryInfoAsZFO(const QString &attachPath = QString(),
-	    const QString &attachFileName = QString(),
-	    const QString &formatString = globPref.delivery_filename_format,
-	    qint64 dmID = -1, const QDateTime &delivTime = QDateTime());
+	void exportSelectedDeliveryInfosAsZFO(void);
 
 	/*!
-	 * @brief Export delivery information as PDF file dialog.
+	 * @brief Export selected delivery information as PDF files.
 	 */
-	void exportDeliveryInfoAsPDF(const QString &attachPath = QString(),
-	    const QString &attachFileName = QString(),
-	    const QString &formatString = globPref.delivery_filename_format,
-	    qint64 dmID = -1, const QDateTime &delivTime = QDateTime());
+	void exportSelectedDeliveryInfosAsPDF(void);
 
 	/*!
-	 * @brief Export selected message envelope as PDF file dialog.
+	 * @brief Export selected message envelope as PDF files.
 	 */
-	void exportMessageEnvelopeAsPDF(const QString &attachPath = QString(),
-	    qint64 dmID = -1, const QDateTime &delivTime = QDateTime());
+	void exportSelectedMessageEnvelopesAsPDF(void);
+
+	/*!
+	 * @brief Sends selected messages as ZFO into default e-mail client.
+	 */
+	void sendMessagesZfoEmail(void);
+
+	/*!
+	 * @brief Sends all attachments of selected messages into default
+	 *     e-mail client.
+	 */
+	void sendAllAttachmentsEmail(void);
+
+	/*!
+	 * @brief Sends selected attachments into default e-mail client.
+	 */
+	void sendAttachmentsEmail(void);
 
 	/*!
 	 * @brief Open selected message in external application.
@@ -548,59 +548,39 @@ private slots:
 	 * @brief Create ZFO file(s) list for import into database.
 	 */
 	void createZFOListForImport(enum ImportZFODialog::ZFOtype zfoType,
-	    enum ImportZFODialog::ZFOaction importType);
+	    enum ImportZFODialog::ZFOaction importType, bool checkOnServer);
 
 	/*!
-	 * @brief Get message type of import ZFO file (message/delivery/unknown).
-	 * Return: -1=error, 0=unknown, 1=message, 2=delivery info
+	 * @brief Collects information about import status.
 	 */
-	int getMessageTypeFromZFO(const QString &file);
+	void collectImportZfoStatus(const QString &fileName, int result,
+	    const QString &resultDesc);
 
 	/*!
-	 * @brief Create account info for ZFO file(s) import into database.
+	 * @brief Create account info list for ZFO files import into database.
+	 *
+	 * @note The user if going to be prompted about missing password.
+	 *
+	 * @param[in] activeOnly Whether to list only active accounts.
+	 * @return List of accounts.
 	 */
-	QList<AccountDataStruct> createAccountInfoForZFOImport(void);
+	QList<Task::AccountDescr> createAccountInfoForZFOImport(
+	    bool activeOnly);
 
 	/*!
 	 * @brief Prepare import ZFO file(s) into database by ZFO type.
 	 */
 	void prepareZFOImportIntoDatabase(const QStringList &files,
-	    enum ImportZFODialog::ZFOtype zfoType);
-
-	/*!
-	 * @brief Import only delivery info ZFO file(s) into database.
-	 */
-	void importDeliveryInfoZFO(
-	    const QList<AccountDataStruct> &accountList,
-	    const QStringList &files,
-	    QList<QPair<QString,QString>> &successFilesList,
-	    QList<QPair<QString,QString>> &existFilesList,
-	    QList<QPair<QString,QString>> &errorFilesList);
-
-	/*!
-	 * @brief Import only message ZFO file(s) into database.
-	 */
-	void importMessageZFO(
-	    const QList<AccountDataStruct> &accountList,
-	    const QStringList &files,
-	    QList<QPair<QString,QString>> &successFilesList,
-	    QList<QPair<QString,QString>> &existFilesList,
-	    QList<QPair<QString,QString>> &errorFilesList);
+	    enum ImportZFODialog::ZFOtype zfoType, bool authenticate);
 
 	/*!
 	 * @brief Show ZFO import notification dialog with results of import.
 	 */
-	void showNotificationDialogWithResult(int filesCnt,
+	void showImportZfoResultDialogue(int filesCnt,
 	    const QList<QPair<QString,QString>> &successFilesList,
 	    const QList<QPair<QString,QString>> &existFilesList,
 	    const QList<QPair<QString,QString>> &errorFilesList);
 
-	/*!
-	 * @brief Check if import ZFO file is/was in ISDS.
-	 */
-	int isImportMsgInISDS(const QString &zfoFile,
-	    QModelIndex accountIndex);
-
 	/*!
 	 * @brief About application dialog.
 	 */
@@ -639,12 +619,22 @@ private slots:
 	/*!
 	 * @brief Set ProgressBar value and Status bar text.
 	 */
-	void setProgressBarFromWorker(QString label, int value);
+	void updateProgressBar(const QString &label, int value);
 
 	/*!
-	 * @brief Refresh AccountList.
+	 * @brief Set Status bar text.
 	 */
-	void refreshAccountListFromWorker(const QString &userName);
+	void updateStatusBarText(const QString &text);
+
+	/*!
+	 * @brief Clear progress bar text.
+	 */
+	void clearProgressBar(void);
+
+	/*!
+	 * @brief Clear status bar text.
+	 */
+	void clearStatusBar(void);
 
 	/*!
 	 * @brief Set and run any actions after main window has been created.
@@ -658,18 +648,6 @@ private slots:
 	void receiveNewDataPath(QString oldDir, QString newDir,
 	    QString action);
 
-	/*!
-	 * @brief Set tablewidget when message download worker is done.
-	 */
-	void postDownloadSelectedMessageAttachments(const QString &userName,
-	    qint64 dmId);
-
-	/*!
-	 * @brief Set info status bar from worker.
-	 */
-	void dataFromWorkerToStatusBarInfo(bool add,
-	    int rt, int rn, int st, int sn);
-
 	/*!
 	 * @brief set message process state into db
 	 */
@@ -700,10 +678,14 @@ private slots:
 	 */
 	void splitMsgDbByYearsSlot(void);
 
+	/*!
+	 * @brief Store last add attachment path.
+	 */
+	void doActionAfterSentMsgSlot(const QString &userName,
+	    const QString &lastDir);
+
 private:
 
-	QThread *m_syncAcntThread;
-	Worker *m_syncAcntWorker;
 	QTimer m_timerSyncAccounts;
 	int m_timeoutSyncAccounts;
 
@@ -711,6 +693,59 @@ private:
 
 	void showStatusTextPermanently(const QString &qStr);
 
+	/*!
+	 * @brief Export message into ZFO file dialogue.
+	 */
+	void exportMessageAsZFO(const QString &attachPath,
+	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
+	    bool askLocation);
+
+	/*!
+	 * @brief Export delivery information as ZFO file dialogue.
+	 */
+	void exportDeliveryInfoAsZFO(const QString &attachPath,
+	    const QString &attachFileName, const QString &formatString,
+	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
+	    bool askLocation);
+
+	/*!
+	 * @brief Export delivery information as PDF file dialogue.
+	 */
+	void exportDeliveryInfoAsPDF(const QString &attachPath,
+	    const QString &attachFileName, const QString &formatString,
+	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
+	    bool askLocation);
+
+	/*!
+	 * @brief Export selected message envelope as PDF file dialogue.
+	 */
+	void exportMessageEnvelopeAsPDF(const QString &attachPath,
+	    const QString &userName, qint64 dmId, QDateTime deliveryTime,
+	    bool askLocation);
+
+	/*!
+	 * @brief Set info status bar from worker.
+	 */
+	void dataFromWorkerToStatusBarInfo(bool add,
+	    int rt, int rn, int st, int sn);
+
+	/*!
+	 * @brief Refresh AccountList.
+	 */
+	void refreshAccountList(const QString &userName);
+
+	/*!
+	 * @brief Set tablewidget when message download worker is done.
+	 */
+	void postDownloadSelectedMessageAttachments(const QString &userName,
+	    qint64 dmId);
+
+	/*!
+	 * @brief Save attachment identified by indexes to file.
+	 */
+	void saveAttachmentToFile(const QModelIndex &messageIndex,
+	    const QModelIndex &attachmentIndex);
+
 	/*!
 	 * @brief Return index for yearly entry with given properties.
 	 */
@@ -769,11 +804,6 @@ private:
 	 */
 	void connectTopMenuBarSlots(void);
 
-	/*!
-	 * @brief Connect top tool-bar buttons to appropriate actions.
-	 */
-	void connectTopToolBarSlots(void);
-
 	/*!
 	 * @brief Connect message-action-bar buttons to appropriate actions.
 	 */
@@ -785,9 +815,14 @@ private:
 	void defaultUiMainWindowSettings(void) const;
 
 	/*!
-	 * @brief Set message action/button to visible.
+	 * @brief Enables message menu actions according to message selection.
 	 */
-	void setMessageActionVisibility(bool action) const;
+	void setMessageActionVisibility(int numSelected) const;
+
+	/*!
+	 * @brief Enables message menu actions according to attachment selection.
+	 */
+	void setAttachmentActionVisibility(int numSelected) const;
 
 	/*!
 	 * @brief Open send message dialog and send message.
@@ -959,16 +994,10 @@ private:
 	qdatovka_error eraseMessage(const QString &userName, qint64 dmId,
 	    const QDateTime &deliveryTime, bool delFromIsds);
 
-	/*!
-	 * @brief Verify message. Compare hash with hash stored in ISDS.
-	 */
-	qdatovka_error verifyMessage(const QString &userName, qint64 dmId,
-	    const QDateTime &deliveryTime);
-
 	/*!
 	 * @brief Authenticate message from ZFO file.
 	 */
-	qdatovka_error authenticateMessageFromZFO(void);
+	int authenticateMessageFromZFO(void);
 
 	/*!
 	 * @brief Show message info for user if connection to ISDS fails.
@@ -1061,9 +1090,22 @@ private:
 	/*!
 	 * @brief Download complete message synchronously
 	 * without worker and thread.
+	 *
+	 * @note Delivery time may change if invalid given.
 	 */
 	bool downloadCompleteMessage(qint64 dmId,
-	    const QDateTime &deliveryTime);
+	    QDateTime &deliveryTime);
+
+	/*!
+	 * @brief Shows notification dialogue and offers downloading of
+	 *     missing message.
+	 *
+	 * @note Delivery time may change if invalid given.
+	 *
+	 * @return True on success.
+	 */
+	bool messageMissingOfferDownload(qint64 dmId,
+	    QDateTime &deliveryTime, const QString &title);
 
 	/*!
 	 * @brief Set read status to messages with given indexes.
@@ -1084,11 +1126,6 @@ private:
 	int showDialogueAboutPwdExpir(const QString &accountName,
 	    const QString &userName, qint64 days, const QDateTime &dateTime);
 
-	/*!
-	 * @brief Get information about remaining PDZ credit.
-	 */
-	QString getPDZCreditFromISDS(void);
-
 	/*!
 	 * @brief Export message with expired time stamp to ZFO.
 	 */
@@ -1150,8 +1187,6 @@ private:
 	                              * Account tree view model. Generated from
 	                              * configuration file.
 	                              */
-	QLineEdit *m_filterLine; /*!< Search filter line object. */
-	QPushButton *m_clearFilterLineButton; /*!< Button object. */
 	SortFilterProxyModel m_messageListProxyModel; /*!<
 	                                                * Used for message
 	                                                * sorting and
@@ -1168,6 +1203,12 @@ private:
 
 	bool m_searchDlgActive; /*!< True if search dialogue is active. */
 
+	QSet<QString> m_zfoFilesToImport; /*!< Set of files to be imported. */
+	int m_numFilesToImport;
+	QList< QPair<QString, QString> > m_importSucceeded,
+	                                 m_importExisted,
+	                                 m_importFailed;
+
 	int m_received_1;
 	int m_received_2;
 	int m_sent_1;
@@ -1182,7 +1223,29 @@ private:
 	QString m_import_zfo_path;
 	bool isMainWindow;
 
-	Ui::MainWindow *ui;
+	/* User interface elements. */
+	Ui::MainWindow *ui; /*!< User interface as generated from ui files. */
+	QLineEdit *mui_filterLine; /*!< Search filter line object. */
+	QPushButton *mui_clearFilterLineButton; /*!< Clear filter button. */
+	QStatusBar *mui_statusBar; /*!< Status bar. */
+	QLabel *mui_statusDbMode; /*!< Database status label. */
+	QLabel *mui_statusOnlineLabel; /*< On-line/off-line status label. */
+	QProgressBar *mui_statusProgressBar; /*!< Progress bar. */
+
+	/*!
+	 * @brief Performs initial user interface initialisation.
+	 */
+	void setUpUi(void);
+
+	/*!
+	 * @brief Adds actions to to tool bar.
+	 */
+	void topToolBarSetUp(void);
+
+	/*!
+	 * @brief Sets action icons.
+	 */
+	void setMenuActionIcons(void);
 };
 
 
