@@ -43,6 +43,9 @@ public:
 	DlgChangePwd(const QString &boxId, const QString &userName,
 	    QWidget *parent = 0);
 
+	static
+	QString generateRandomString(int stringLength);
+
 private slots:
 	void generatePassword(void);
 	void showHidePasswordLine(void);
@@ -55,9 +58,6 @@ private:
 	QTimer *pingTimer;
 	void initPwdChangeDialog(void);
 
-	static
-	QString generateRandomString(void);
-
 	static
 	const QString possibleCharacters;
 	static
