@@ -28,6 +28,8 @@
 #include <QTextDocument>
 
 #include "dlg_correspondence_overview.h"
+#include "src/io/filesystem.h"
+#include "src/settings/preferences.h"
 
 
 /* ========================================================================= */
@@ -575,10 +577,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				        mId.deliveryTime);
 				Q_ASSERT(0 != messageDb);
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.message_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -607,10 +609,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.message_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -646,10 +648,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.delivery_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -678,10 +680,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.delivery_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -716,10 +718,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.message_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -747,10 +749,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.message_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -784,10 +786,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.delivery_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
@@ -816,10 +818,10 @@ void DlgCorrespondenceOverview::exportData(void)
 				Q_ASSERT(0 != messageDb);
 
 				entry = messageDb->msgsGetAdditionalFilenameEntry(mId.dmId);
-				fileName = createFilenameFromFormatString(
+				fileName = fileNameFromFormat(
 				    globPref.delivery_filename_format,
-				    QString::number(mId.dmId), m_dbId,
-				    m_userName, "", entry.dmDeliveryTime,
+				    mId.dmId, m_dbId, m_userName, "",
+				    entry.dmDeliveryTime,
 				    entry.dmAcceptanceTime, entry.dmAnnotation,
 				    entry.dmSender);
 
