@@ -30,12 +30,10 @@
 #include "src/models/accounts_model.h"
 
 
-DlgCreateAccount::DlgCreateAccount(QTreeView &accountList,
-    QModelIndex acntTopIdx, Action action,
-    QWidget *parent)
+DlgCreateAccount::DlgCreateAccount(const AccountModel::SettingsMap &accountInfo,
+    Action action, QWidget *parent)
     : QDialog(parent),
-    m_accountList(accountList),
-    m_acntTopIdx(acntTopIdx),
+    m_accountInfo(accountInfo),
     m_action(action),
     m_loginmethod(0),
     m_certPath("")
@@ -89,72 +87,63 @@ void DlgCreateAccount::setCurrentAccountData(void)
 /* ========================================================================= */
 {
 	int itemindex;
-	QModelIndex index;
-	AccountModel *model = dynamic_cast<AccountModel *>(m_accountList.model());
 
-	if (ACT_EDIT == m_action) {
-		index = m_accountList.currentIndex();
-	} else  {
-		index = m_acntTopIdx;
+	if (m_accountInfo.userName().isEmpty()) {
+		Q_ASSERT(0);
+		return;
 	}
 
-	const QStandardItem *item = model->itemFromIndex(index);
-	const QStandardItem *itemTop = AccountModel::itemTop(item);
-
-	const AccountModel::SettingsMap &itemSettings =
-	    itemTop->data(ROLE_ACNT_CONF_SETTINGS).toMap();
-
 	switch (m_action) {
 	case ACT_EDIT:
 		this->setWindowTitle(tr("Update account") + " "
-		    + itemTop->text());
-		this->accountLineEdit->setText(itemTop->text());
-		this->usernameLineEdit->setText(itemSettings.userName());
+		    + m_accountInfo.accountName());
+		this->accountLineEdit->setText(m_accountInfo.accountName());
+		this->usernameLineEdit->setText(m_accountInfo.userName());
 		this->usernameLineEdit->setEnabled(false);
 		break;
 	case ACT_PWD:
 		this->setWindowTitle(tr("Enter password for account") + " "
-		    + itemTop->text());
-		this->accountLineEdit->setText(itemTop->text());
+		    + m_accountInfo.accountName());
+		this->accountLineEdit->setText(m_accountInfo.accountName());
 		this->accountLineEdit->setEnabled(false);
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
-		this->usernameLineEdit->setText(itemSettings.userName());
+		this->usernameLineEdit->setText(m_accountInfo.userName());
 		this->testAccountCheckBox->setEnabled(false);
 		this->usernameLineEdit->setEnabled(false);
 		this->addCertificateButton->setEnabled(false);
 		break;
 	case ACT_CERT:
 		this->setWindowTitle(tr("Set certificate for account") + " "
-		    + itemTop->text());
-		this->accountLineEdit->setText(itemTop->text());
+		    + m_accountInfo.accountName());
+		this->accountLineEdit->setText(m_accountInfo.accountName());
 		this->accountLineEdit->setEnabled(false);
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
-		this->usernameLineEdit->setText(itemSettings.userName());
+		this->usernameLineEdit->setText(m_accountInfo.userName());
 		this->testAccountCheckBox->setEnabled(false);
 		this->usernameLineEdit->setEnabled(false);
 		this->passwordLineEdit->setEnabled(false);
 		break;
 	case ACT_CERTPWD:
 		this->setWindowTitle(tr("Enter password/certificate for account")
-		    + " " + itemTop->text());
-		this->accountLineEdit->setText(itemTop->text());
+		    + " " + m_accountInfo.accountName());
+		this->accountLineEdit->setText(m_accountInfo.accountName());
 		this->accountLineEdit->setEnabled(false);
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
-		this->usernameLineEdit->setText(itemSettings.userName());
+		this->usernameLineEdit->setText(m_accountInfo.userName());
 		this->testAccountCheckBox->setEnabled(false);
 		this->usernameLineEdit->setEnabled(false);
 		break;
 	case ACT_IDBOX:
 		this->setWindowTitle(tr("Enter ID of your databox for account")
-		    + " " + itemTop->text());
-		this->accountLineEdit->setText(itemTop->text());
+		    + " " + m_accountInfo.accountName());
+		this->accountLineEdit->setText(m_accountInfo.accountName());
 		this->accountLineEdit->setEnabled(false);
 		this->infoLabel->setEnabled(false);
 		this->loginmethodComboBox->setEnabled(false);
-		this->usernameLineEdit->setText(itemSettings.userName());
+		this->usernameLineEdit->setText(m_accountInfo.userName());
 		this->testAccountCheckBox->setEnabled(false);
 		this->addCertificateButton->setEnabled(false);
 		this->passwordLineEdit->setEnabled(false);
@@ -165,7 +154,7 @@ void DlgCreateAccount::setCurrentAccountData(void)
 		break;
 	}
 
-	const QString login_method = itemSettings.loginMethod();
+	const QString login_method = m_accountInfo.loginMethod();
 	if (LIM_USERNAME == login_method) {
 		itemindex = USER_NAME;
 	} else if (LIM_CERT == login_method) {
@@ -181,17 +170,17 @@ void DlgCreateAccount::setCurrentAccountData(void)
 	this->loginmethodComboBox->setCurrentIndex(itemindex);
 	setActiveButton(itemindex);
 
-	this->passwordLineEdit->setText(itemSettings.password());
-	this->testAccountCheckBox->setChecked(itemSettings.isTestAccount());
-	this->rememberPswcheckBox->setChecked(itemSettings.rememberPwd());
-	this->synchroCheckBox->setChecked(itemSettings.syncWithAll());
+	this->passwordLineEdit->setText(m_accountInfo.password());
+	this->testAccountCheckBox->setChecked(m_accountInfo.isTestAccount());
+	this->rememberPswcheckBox->setChecked(m_accountInfo.rememberPwd());
+	this->synchroCheckBox->setChecked(m_accountInfo.syncWithAll());
 
-	if (!itemSettings.p12File().isEmpty()) {
+	if (!m_accountInfo.p12File().isEmpty()) {
 		this->addCertificateButton->setText(QDir::
-		    toNativeSeparators(itemSettings.p12File()));
+		    toNativeSeparators(m_accountInfo.p12File()));
 		this->addCertificateButton->setIcon(QIcon(ICON_3PARTY_PATH +
 		QString("key_16.png")));
-		m_certPath = QDir::toNativeSeparators(itemSettings.p12File());
+		m_certPath = QDir::toNativeSeparators(m_accountInfo.p12File());
 	}
 
 	checkInputFields();
@@ -258,22 +247,21 @@ void DlgCreateAccount::setActiveButton(int itemindex)
 		this->passwordLabel->setEnabled(false);
 		this->passwordLineEdit->setEnabled(false);
 		this->rememberPswcheckBox->setEnabled(false);
-		this->usernameLineEdit->setEnabled(false);
 	} else if (itemindex == USER_CERTIFICATE) {
 		this->certificateLabel->setEnabled(true);
 		this->addCertificateButton->setEnabled(true);
 		this->passwordLabel->setEnabled(true);
 		this->passwordLineEdit->setEnabled(true);
 		this->rememberPswcheckBox->setEnabled(true);
-		this->usernameLineEdit->setEnabled(true);
 	} else {
 		this->certificateLabel->setEnabled(false);
 		this->addCertificateButton->setEnabled(false);
 		this->passwordLabel->setEnabled((true));
 		this->passwordLineEdit->setEnabled((true));
 		this->rememberPswcheckBox->setEnabled(true);
-		this->usernameLineEdit->setEnabled(false);
 	}
+	this->usernameLineEdit->setEnabled(m_action == ACT_ADDNEW);
+
 	m_loginmethod = itemindex;
 	checkInputFields();
 }
@@ -288,37 +276,23 @@ void DlgCreateAccount::saveAccount(void)
 {
 	debugSlotCall();
 
-	/* get current account model */
-	AccountModel *model = dynamic_cast<AccountModel*>(
-	    m_accountList.model());
-	QModelIndex index;
-	QStandardItem *itemTop;
 	AccountModel::SettingsMap itemSettings;
+	const QString userName(m_accountInfo.userName());
 
 	/* set account index, itemTop and map itemSettings for account */
 	switch (m_action) {
 	case ACT_ADDNEW:
-		itemTop = NULL;
 		break;
 	case ACT_EDIT:
-		index = m_accountList.currentIndex();
-		Q_ASSERT(index.isValid());
-		itemTop = AccountModel::itemTop(model->itemFromIndex(index));
-		Q_ASSERT(0 != itemTop);
-		itemSettings = itemTop->data(ROLE_ACNT_CONF_SETTINGS).toMap();
-		break;
 	case ACT_PWD:
 	case ACT_CERT:
 	case ACT_CERTPWD:
 	case ACT_IDBOX:
-		index = m_acntTopIdx;
-		Q_ASSERT(index.isValid());
-		itemTop = AccountModel::itemTop(model->itemFromIndex(index));
-		Q_ASSERT(0 != itemTop);
-		itemSettings = itemTop->data(ROLE_ACNT_CONF_SETTINGS).toMap();
+		Q_ASSERT(!userName.isEmpty());
+		Q_ASSERT(userName == this->usernameLineEdit->text());
+		itemSettings = AccountModel::globAccounts[userName];
 		break;
 	default:
-		itemTop = NULL;
 		Q_ASSERT(0);
 		break;
 	}
@@ -364,8 +338,13 @@ void DlgCreateAccount::saveAccount(void)
 	case ACT_CERT:
 	case ACT_CERTPWD:
 	case ACT_IDBOX:
-		itemTop->setText(this->accountLineEdit->text());
-		itemTop->setData(itemSettings, ROLE_ACNT_CONF_SETTINGS);
+		/* TODO -- Update account model according to settings. */
+		AccountModel::globAccounts[userName] = itemSettings;
+		/*
+		 * Catching the following signal is required only when
+		 * ACT_EDIT was enabled.
+		 */
+		emit changedAccountProperties(userName);
 		/* TODO -- Save/update related account DB entry? */
 		break;
 	case ACT_ADDNEW:
