@@ -28,6 +28,9 @@
 #include "dlg_ds_search.h"
 #include "src/io/isds_sessions.h"
 #include "src/views/table_home_end_filter.h"
+#include "src/views/table_space_selection_filter.h"
+#include "src/worker/pool.h"
+#include "src/worker/task_search_owner.h"
 
 
 DlgDsSearch::DlgDsSearch(Action action, QTableWidget *recipientTableWidget,
@@ -131,6 +134,8 @@ void DlgDsSearch::initSearchWindow(void)
 
 	this->resultsTableWidget->installEventFilter(
 	    new TableHomeEndFilter(this));
+	this->resultsTableWidget->installEventFilter(
+	    new TableSpaceSelectionFilter(this));
 
 	pingTimer = new QTimer(this);
 	pingTimer->start(DLG_ISDS_KEEPALIVE_MS);
@@ -338,8 +343,17 @@ void DlgDsSearch::searchDataBox(void)
 		goto fail;
 	}
 
-	isds_error status;
-	status = isdsSearch(&boxes, m_userName, ownerInfo);
+	int status;
+	TaskSearchOwner *task;
+
+	task = new (std::nothrow) TaskSearchOwner(m_userName, ownerInfo);
+	task->setAutoDelete(false);
+	globWorkPool.runSingle(task);
+
+	status = task->m_isdsRetError;
+	boxes = task->m_results; task->m_results = NULL;
+
+	delete task;
 	isds_DbOwnerInfo_free(&ownerInfo);
 
 	session = isdsSessions.session(m_userName);
@@ -389,7 +403,6 @@ void DlgDsSearch::searchDataBox(void)
 		this->resultsTableWidget->setEnabled(true);
 		isds_DbOwnerInfo *item = (isds_DbOwnerInfo *) box->data;
 		Q_ASSERT(0 != item);
-		qDebug() << item->dbID;
 		QVector<QString> contact;
 		contact.append(item->dbID);
 
