@@ -32,13 +32,15 @@
 #include <QTimeZone>
 #include <QUrl>
 
-#include "src/common.h"
 #include "src/crypto/crypto_funcs.h"
+#include "src/gui/dlg_import_zfo.h"
 #include "src/gui/dlg_signature_detail.h"
 #include "src/gui/dlg_view_zfo.h"
 #include "src/io/dbs.h"
+#include "src/io/filesystem.h"
+#include "src/io/isds_sessions.h"
 #include "src/log/log.h"
-#include "src/gui/dlg_import_zfo.h"
+#include "src/settings/preferences.h"
 #include "src/views/table_home_end_filter.h"
 
 
@@ -50,7 +52,7 @@ DlgViewZfo::DlgViewZfo(const QString &zfoFileName, QWidget *parent)
 /* ========================================================================= */
     : QDialog(parent),
     m_message(NULL),
-    m_attachmentModel(0)
+    m_attachmentModel(this)
 {
 	setupUi(this);
 
@@ -83,7 +85,7 @@ DlgViewZfo::DlgViewZfo(const QString &zfoFileName, QWidget *parent)
 	} else {
 		this->attachmentTable->setEnabled(true);
 		this->attachmentTable->show();
-		m_attachmentModel.setModelData(m_message);
+		m_attachmentModel.setMessage(m_message);
 		envelopeTextEdit->setHtml(
 		    messageDescriptionHtml(m_attachmentModel.rowCount(),
 		        m_message->raw, m_message->raw_length,
@@ -93,7 +95,15 @@ DlgViewZfo::DlgViewZfo(const QString &zfoFileName, QWidget *parent)
 
 		/* Attachment list. */
 		attachmentTable->setModel(&m_attachmentModel);
-		attachmentTable->resizeColumnToContents(0);
+		/* First three columns contain hidden data. */
+		attachmentTable->setColumnHidden(DbFlsTblModel::ATTACHID_COL,
+		    true);
+		attachmentTable->setColumnHidden(DbFlsTblModel::MSGID_COL,
+		    true);
+		attachmentTable->setColumnHidden(DbFlsTblModel::CONTENT_COL,
+		    true);
+		attachmentTable->resizeColumnToContents(
+		    DbFlsTblModel::FNAME_COL);
 
 		attachmentTable->setContextMenuPolicy(Qt::CustomContextMenu);
 		connect(attachmentTable, SIGNAL(customContextMenuRequested(QPoint)),
@@ -167,8 +177,6 @@ void DlgViewZfo::attachmentItemDoubleClicked(const QModelIndex &index)
 {
 	(void) index;
 
-	//qDebug() << "Attachment double clicked.";
-
 	openSelectedAttachment();
 }
 
@@ -234,8 +242,15 @@ void DlgViewZfo::saveSelectedAttachmentToFile(void)
 		return;
 	}
 
+	QModelIndex dataIndex = selectedIndex.sibling(selectedIndex.row(),
+	    DbFlsTblModel::CONTENT_COL);
+	if (!dataIndex.isValid()) {
+		Q_ASSERT(0);
+		return;
+	}
+
 	QByteArray data =
-	    m_attachmentModel.attachmentData(selectedIndex.row());
+	    QByteArray::fromBase64(dataIndex.data().toByteArray());
 
 	if (WF_SUCCESS != writeFile(fileName, data)) {
 		QMessageBox::warning(this,
@@ -288,7 +303,15 @@ void DlgViewZfo::saveSelectedAttachmentsIntoDirectory(void)
 
 		fileName = newDir + QDir::separator() + fileName;
 
-		QByteArray data = m_attachmentModel.attachmentData(idx.row());
+		QModelIndex dataIndex = idx.sibling(idx.row(),
+		    DbFlsTblModel::CONTENT_COL);
+		if (!dataIndex.isValid()) {
+			Q_ASSERT(0);
+			continue;
+		}
+
+		QByteArray data =
+		    QByteArray::fromBase64(dataIndex.data().toByteArray());
 
 		if (WF_SUCCESS != writeFile(fileName, data)) {
 			unsuccessfullFiles.append(fileName);
@@ -342,12 +365,19 @@ void DlgViewZfo::openSelectedAttachment(void)
 	/* TODO -- Add message id into file name? */
 	QString fileName = TMP_ATTACHMENT_PREFIX + attachName;
 
+	QModelIndex dataIndex = selectedIndex.sibling(selectedIndex.row(),
+	    DbFlsTblModel::CONTENT_COL);
+	if (!dataIndex.isValid()) {
+		Q_ASSERT(0);
+		return;
+	}
+
 	QByteArray data =
-	    m_attachmentModel.attachmentData(selectedIndex.row());
+	    QByteArray::fromBase64(dataIndex.data().toByteArray());
 
 	fileName = writeTemporaryFile(fileName, data);
 	if (!fileName.isEmpty()) {
-		QDesktopServices::openUrl(QUrl("file:///" + fileName));
+		QDesktopServices::openUrl(QUrl(URL_FILE_PREFIX + fileName));
 		/* TODO -- Handle openUrl() return value. */
 	} else {
 		QMessageBox::warning(this,
@@ -458,7 +488,7 @@ QString DlgViewZfo::deliveryDescriptionHtml(const void *msgDER,
 
 /* ========================================================================= */
 bool DlgViewZfo::envelopeHeaderDescriptionHtml(QString &html,
-    const isds_envelope *envelope)
+    const struct isds_envelope *envelope)
 /* ========================================================================= */
 {
 	if (NULL == envelope) {
@@ -490,13 +520,15 @@ bool DlgViewZfo::envelopeHeaderDescriptionHtml(QString &html,
 	html += "<h3>" + tr("Status") + "</h3>";
 
 	html += strongAccountInfoLine(tr("Delivery time"),
-	    dateTimeStrFromDbFormat(
-	        timevalToDbFormat(envelope->dmDeliveryTime),
-	        dateTimeDisplayFormat));
+	    (NULL != envelope->dmDeliveryTime) ?
+	        dateTimeStrFromDbFormat(
+	            timevalToDbFormat(envelope->dmDeliveryTime),
+	            dateTimeDisplayFormat) : "");
 	html += strongAccountInfoLine(tr("Acceptance time"),
-	    dateTimeStrFromDbFormat(
-	        timevalToDbFormat(envelope->dmAcceptanceTime),
-	        dateTimeDisplayFormat));
+	    (NULL != envelope->dmAcceptanceTime) ?
+	        dateTimeStrFromDbFormat(
+	            timevalToDbFormat(envelope->dmAcceptanceTime),
+	            dateTimeDisplayFormat) : "");
 
 	QString statusString;
 	if (NULL != envelope->dmMessageStatus) {
@@ -574,7 +606,8 @@ QModelIndex DlgViewZfo::selectedAttachmentIndex(void) const
 		return QModelIndex();
 	}
 
-	return selectedIndex.sibling(selectedIndex.row(), FNAME_COL);
+	return selectedIndex.sibling(selectedIndex.row(),
+	    DbFlsTblModel::FNAME_COL);
 }
 
 
