@@ -27,9 +27,7 @@
 
 #include <QDialog>
 
-#include "src/common.h"
-#include "src/io/isds_sessions.h"
-#include "src/models/attachment_model.h"
+#include "src/models/files_model.h"
 #include "ui_dlg_view_zfo.h"
 
 /*!
@@ -136,7 +134,7 @@ private:
 	 */
 	static
 	bool envelopeHeaderDescriptionHtml(QString &html,
-	    const isds_envelope *envelope);
+	    const struct isds_envelope *envelope);
 
 	/*!
 	 * @brief Generates footer description according to the supplied
@@ -162,7 +160,7 @@ private:
 	 */
 
 	int m_zfoType;
-	AttachmentModel m_attachmentModel; /*!< Attachment model. */
+	DbFlsTblModel m_attachmentModel; /*!< Attachment model. */
 };
 
 #endif /* _DLG_VIEW_ZFO_H_ */
