@@ -32,7 +32,7 @@
 #include "isds_sessions.h"
 #include "src/gui/dlg_import_zfo.h"
 #include "src/log/log.h"
-
+#include "src/settings/preferences.h"
 
 
 GlobIsdsSessions isdsSessions;
@@ -830,32 +830,6 @@ fail:
 }
 
 
-/* ========================================================================= */
-/*
- * Search DataBoxes.
- */
-isds_error isdsSearch(struct isds_list **result, const QString &userName,
-    const struct isds_DbOwnerInfo *ownerInfo)
-/* ========================================================================= */
-{
-	isds_error ret = IE_ERROR;
-
-	if ((NULL == result) || (NULL == ownerInfo)) {
-		return IE_ERROR;
-	}
-
-	struct isds_ctx *session = isdsSessions.session(userName);
-	if (NULL == session) {
-		Q_ASSERT(0);
-		return IE_ERROR;
-	}
-
-	ret = isds_FindDataBox(session, ownerInfo, result);
-
-	qDebug() << ret << isds_strerror(ret);
-	return ret;
-}
-
 /* ========================================================================= */
 /*
  * Create DbUserInfo structure and Search DataBoxes.
