@@ -36,7 +36,6 @@
 #include <QMap>
 #include <QString>
 
-#include "src/common.h"
 #include "src/models/accounts_model.h"
 
 
@@ -180,13 +179,6 @@ struct isds_DbOwnerInfo * isds_DbOwnerInfo_createConsume(const QString &dbID,
     bool dbEffectiveOVM, bool dbOpenAddressing);
 
 
-/*!
- * @brief Search DataBoxes.
- */
-isds_error isdsSearch(struct isds_list **result, const QString &userName,
-    const struct isds_DbOwnerInfo *ownerInfo);
-
-
 /*!
  * @brief Create DbUserInfo structure.
  *
