@@ -32,7 +32,6 @@
 #include <QtWidgets>
 
 #include "src/cli/cli.h"
-#include "src/common.h"
 #include "src/crypto/crypto.h"
 #include "src/crypto/crypto_threads.h"
 #include "src/crypto/crypto_funcs.h"
@@ -40,9 +39,13 @@
 #include "src/gui/dlg_view_zfo.h"
 #include "src/io/db_tables.h"
 #include "src/io/file_downloader.h"
+#include "src/io/filesystem.h"
 #include "src/io/message_db.h"
 #include "src/io/message_db_set_container.h"
 #include "src/log/log.h"
+#include "src/settings/proxy.h"
+#include "src/single/single_instance.h"
+#include "src/worker/pool.h"
 
 #define CONF_SUBDIR_OPT "conf-subdir"
 #define LOAD_CONF_OPT "load-conf"
@@ -435,6 +438,20 @@ int main(int argc, char *argv[])
 
 	QStringList cmdLineFileNames = parser.positionalArguments();
 
+	/* Ensure only one instance with given configuration file. */
+	SingleInstance singleInstance(globPref.loadConfPath());
+	if (singleInstance.existsInSystem()) {
+		logErrorNL("%s",
+		    "Another application already uses same configuration.");
+		logInfo("%s\n", "Notifying key owner and exiting");
+		if (!singleInstance.sendMessage(
+		        SingleInstance::msgRaiseMainWindow)) {
+			logErrorNL("%s",
+			    "Could not send message to owner.");
+		}
+		return EXIT_FAILURE;
+	}
+
 	qint64 start, stop, diff;
 	start = QDateTime::currentMSecsSinceEpoch();
 	logInfo("Starting at %lld.%03lld .\n", start / 1000, start % 1000);
@@ -462,7 +479,7 @@ int main(int argc, char *argv[])
 		    globPref.saveConfPath().toUtf8().constData());
 
 		/* Change "\" to "/" */
-		fixBackSlashesInFile(globPref.loadConfPath());
+		confFileFixBackSlashes(globPref.loadConfPath());
 	}
 
 	{
@@ -570,6 +587,10 @@ int main(int argc, char *argv[])
 		AccountModel::globAccounts.loadFromSettings(settings);
 	}
 
+	/* Start worker threads. */
+	globWorkPool.start();
+	logInfo("%s\n", "Worker pool started.");
+
 	QStringList inOptions = parser.optionNames();
 	QStringList serList;
 
@@ -599,6 +620,12 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	/* Wait until all threads finished. */
+	logInfo("%s\n", "Waiting for pending worker threads.");
+	globWorkPool.wait();
+	globWorkPool.stop();
+	logInfo("%s\n", "All worker threads finished");
+
 	stop = QDateTime::currentMSecsSinceEpoch();
 	diff = stop - start;
 	logInfo("Stopping at %lld.%03lld; ran for %lld.%03lld seconds.\n",
